//-----------------------------------------------------------------------------
// File: mp_wrap.cpp
//
// Copyright (c) Astralax. All rights reserved.
// Author: Sedov Alexey
//-----------------------------------------------------------------------------

#include "mp_wrap.h"

#pragma warning ( disable : 4996)

// -------------

// eng: Class storing the textural atlas
// rus: ,    
MP_Atlas_WRAP::MP_Atlas_WRAP(int width, int height, const char* file) : MP_Atlas(width, height, file)
{
	LPDIRECT3DDEVICE9 device_3d=device->GetDevice3D();

	if (file_name.empty())
	{
		if (width>0 && height>0)
		{
			device_3d->CreateTexture(atlas_width,atlas_height,1,D3DUSAGE_RENDERTARGET,D3DFMT_A8R8G8B8,D3DPOOL_DEFAULT,&texture,NULL);

			// cleaning up of created texture
			// rus:   
			IDirect3DSurface9* back=NULL;
			HRESULT hr=device_3d->GetRenderTarget(0,&back);

			IDirect3DSurface9* dest=NULL;
			texture->GetSurfaceLevel(0,&dest);
			hr=device_3d->SetRenderTarget(0,dest);
			device_3d->Clear(0, NULL, D3DCLEAR_TARGET, 0x0, 0.0f, 0);
			dest->Release();

			device_3d->SetRenderTarget(0,back);

			back->Release();
		}
	}
	else
	{
		std::string texture_file=MP->GetPathToTexture();
		texture_file+=file_name.c_str();
		const wchar_t* texture_file_utf16=(const wchar_t*)Magic_UTF8to16((const unsigned char*)texture_file.c_str());
		D3DXCreateTextureFromFileW(device_3d, texture_file_utf16, &texture);
	}
}

// eng: Destroy atlas texture
// rus:   
void MP_Atlas_WRAP::Destroy()
{
	if (texture)
	{
		texture->Release();
		texture=NULL;
	}
}

// eng: Loading of frame texture
// rus:   
void MP_Atlas_WRAP::LoadTexture(const MAGIC_CHANGE_ATLAS* c)
{
	LPDIRECT3DDEVICE9 device_3d=device->GetDevice3D();

	DWORD state_alphaenable, state_zenable, state_zwriteenable;
	device_3d->GetRenderState(D3DRS_ALPHABLENDENABLE, &state_alphaenable);
	device_3d->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
	device_3d->GetRenderState(D3DRS_ZENABLE, &state_zenable);
	device_3d->SetRenderState(D3DRS_ZENABLE, FALSE);
	device_3d->GetRenderState(D3DRS_ZWRITEENABLE, &state_zwriteenable);
	device_3d->SetRenderState(D3DRS_ZWRITEENABLE, FALSE);

	IDirect3DSurface9* dest=NULL;
	texture->GetSurfaceLevel(0,&dest);

	IDirect3DSurface9* back=NULL;
	HRESULT hr=device_3d->GetRenderTarget(0,&back);

	hr=device_3d->SetRenderTarget(0,dest);

	device_3d->BeginScene();

	LPDIRECT3DTEXTURE9 texture_frame=NULL;

	D3DXIMAGE_INFO info;
	if (c->data)
	{
		// eng: loading from memory
		// rus:   
		hr=D3DXCreateTextureFromFileInMemoryEx(device_3d,c->data,c->length,0,0,D3DX_DEFAULT,0,D3DFMT_A8R8G8B8,D3DPOOL_DEFAULT,D3DX_FILTER_NONE,D3DX_DEFAULT,0,&info,NULL,&texture_frame);
	}
	else
	{
		// eng: loading from file
		// rus:   
		std::string texture_file=MP->GetPathToTexture();
		texture_file+=c->file;
		const wchar_t* texture_file_utf16=(const wchar_t*)Magic_UTF8to16((const unsigned char*)texture_file.c_str());
		hr=D3DXCreateTextureFromFileExW(device_3d,texture_file_utf16,0,0,D3DX_DEFAULT,0,D3DFMT_A8R8G8B8,D3DPOOL_DEFAULT,D3DX_FILTER_NONE,D3DX_DEFAULT,0,&info,NULL,&texture_frame);
	}

	ID3DXSprite* sprite=NULL;

	D3DXCreateSprite(device_3d, &sprite);

	RECT rect_from;
	rect_from.left=0;
	rect_from.top=0;
	rect_from.right=info.Width;
	rect_from.bottom=info.Height;

	float scale_x=((float)c->width)/((float)info.Width);
	float scale_y=((float)c->height)/((float)info.Height);

	D3DXMATRIX scale_matrix;
	D3DXMatrixScaling(&scale_matrix,scale_x,scale_y,1);
	sprite->SetTransform(&scale_matrix);

	sprite->Begin(0);
	hr=sprite->Draw(texture_frame,  // rus:  
		&rect_from,						// rus:  
		NULL,
		&D3DXVECTOR3((float)c->x/scale_x, (float)c->y/scale_y, 0.f), 0xffffffff);

	sprite->End();

	sprite->Release();
	sprite=NULL;

	texture_frame->Release();
	texture_frame=NULL;

	dest->Release();

	device_3d->SetRenderTarget(0,back);

	back->Release();

	device_3d->EndScene();

	device_3d->SetRenderState(D3DRS_ZENABLE, state_zenable);
	device_3d->SetRenderState(D3DRS_ZWRITEENABLE, state_zwriteenable);
	device_3d->SetRenderState(D3DRS_ALPHABLENDENABLE, state_alphaenable);
}

// eng: Cleaning up of rectangle
// rus:  
void MP_Atlas_WRAP::CleanRectangle(const MAGIC_CHANGE_ATLAS* c)
{
	LPDIRECT3DDEVICE9 device_3d=device->GetDevice3D();

	IDirect3DSurface9* back=NULL;
	HRESULT hr=device_3d->GetRenderTarget(0,&back);

	IDirect3DSurface9* dest=NULL;
	texture->GetSurfaceLevel(0,&dest);
	hr=device_3d->SetRenderTarget(0,dest);
	D3DRECT rect;
	rect.x1=c->x;
	rect.y1=c->y;
	rect.x2=c->x+c->width;
	rect.y2=c->y+c->height;
	device_3d->Clear(1, &rect, D3DCLEAR_TARGET, 0x0, 0.0f, 0);
	dest->Release();

	device_3d->SetRenderTarget(0,back);

	back->Release();
}


// --------------------------------------------------------------------------------

MP_LAYOUT_WRAP::MP_LAYOUT_WRAP() : MP_LAYOUT()
{
	declaration=NULL;
}

MP_LAYOUT_WRAP::~MP_LAYOUT_WRAP()
{
	if (declaration)
	{
		declaration->Release();
		declaration=NULL;
	}
}

void MP_LAYOUT_WRAP::Generate(MAGIC_VERTEX_FORMAT* format, MP_Device* device)
{
	int t=0;
	int n=0;
	D3DVERTEXELEMENT9 decl[50];

	// 
	D3DVERTEXELEMENT9* d=&(decl[t]);
	d->Stream=0;
	d->Offset=n;
	d->Method=D3DDECLMETHOD_DEFAULT;
	d->UsageIndex=0;

	if (((MP_Device_WRAP*)device)->GetAdditionalVerticesPosition()==2)
	{
		// 2D
		d->Type=D3DDECLTYPE_FLOAT4;
		d->Usage=D3DDECLUSAGE_POSITIONT;
		n=16;
	}
	else
	{
		// 3D
		d->Type=D3DDECLTYPE_FLOAT3;
		d->Usage=D3DDECLUSAGE_POSITION;
		n=12;
	}

	t++;

	if (format->attributes & MAGIC_ATTRIBUTE_COLOR)
	{
		// 
		d=&(decl[t]);
		d->Stream=0;
		d->Offset=n;
		d->Method=D3DDECLMETHOD_DEFAULT;
		d->UsageIndex=0;
		d->Type=D3DDECLTYPE_D3DCOLOR;
		d->Usage=D3DDECLUSAGE_COLOR;
		n+=4;

		t++;
	}

	for (int i=0;i<format->UVs;i++)
	{
		//  
		d=&(decl[t]);
		d->Stream=0;
		d->Offset=n;
		d->Method=D3DDECLMETHOD_DEFAULT;
		d->UsageIndex=i;
		d->Type=D3DDECLTYPE_FLOAT2;
		d->Usage=D3DDECLUSAGE_TEXCOORD;
		n+=8;

		t++;
	}

	//  
	d=&(decl[t]);
	d->Stream=0xff;
	d->Offset=0;
	d->Method=0;
	d->UsageIndex=0;
	d->Type=D3DDECLTYPE_UNUSED;
	d->Usage=0;

	((MP_Device_WRAP*)device)->GetDevice3D()->CreateVertexDeclaration(decl, &declaration);
}

// --------------------------------------------------------------------------------

MP_VERTEX_BUFFER_WRAP::MP_VERTEX_BUFFER_WRAP() : MP_BUFFER()
{
	buffer=NULL;

	filled_length=filled_stride=0;
}

void MP_VERTEX_BUFFER_WRAP::Create(int new_length, MP_Device* device)
{
	MP_BUFFER::Create(new_length, device);
	((MP_Device_WRAP*)device)->GetDevice3D()->CreateVertexBuffer(new_length, D3DUSAGE_DYNAMIC | D3DUSAGE_WRITEONLY, 0, D3DPOOL_DEFAULT, &buffer, NULL);

	filled_length=filled_stride=0;
}

void MP_VERTEX_BUFFER_WRAP::Destroy()
{
	MP_BUFFER::Destroy();
	if (buffer)
	{
		buffer->Release();
		buffer=NULL;
	}
}

void* MP_VERTEX_BUFFER_WRAP::Map(int stride, MP_Device* device)
{
	void* locked_buffer;
	buffer->Lock(0, length, &locked_buffer, 0);

	int additional_vertex_position=((MP_Device_WRAP*)device)->GetAdditionalVerticesPosition();
	if (additional_vertex_position)
	{
		if (filled_length<length || filled_stride!=stride)
		{
			filled_length=length;
			filled_stride=stride;

			int len=length/stride;
			int step=stride/4;
			float* pos_buf=(float*)locked_buffer;
			pos_buf+=2; //   x,y

			if (additional_vertex_position==2)
			{
				for (int i=0;i<len;i++)
				{
					int index=i*step;
					pos_buf[index]=0.f; // Z
					pos_buf[index+1]=1.f; // RHW
				}
			}
			else
			{
				for (int i=0;i<len;i++)
				{
					int index=i*step;
					pos_buf[index]=0.f; // Z
				}
			}
		}
	}

	return locked_buffer;
}

void MP_VERTEX_BUFFER_WRAP::Unmap(MP_Device* device)
{
	buffer->Unlock();
}

void MP_VERTEX_BUFFER_WRAP::Set(MAGIC_VERTEX_FORMAT* format, int stride, MP_Device* device)
{
	((MP_Device_WRAP*)device)->GetDevice3D()->SetStreamSource(0, buffer, 0, stride);
}

// -------------------------------

MP_INDEX_BUFFER_WRAP::MP_INDEX_BUFFER_WRAP() : MP_BUFFER()
{
	buffer=NULL;
}

void MP_INDEX_BUFFER_WRAP::Create(int new_length, MP_Device* device)
{
	MP_BUFFER::Create(new_length, device);

	#ifdef INDEX_BUFFER_32_WRAP
	_D3DFORMAT index_size=D3DFMT_INDEX32;
	#else
	_D3DFORMAT index_size=D3DFMT_INDEX16;
	#endif

	((MP_Device_WRAP*)device)->GetDevice3D()->CreateIndexBuffer(new_length, D3DUSAGE_DYNAMIC | D3DUSAGE_WRITEONLY, index_size, D3DPOOL_DEFAULT, &buffer, NULL);
}

void MP_INDEX_BUFFER_WRAP::Destroy()
{
	MP_BUFFER::Destroy();
	if (buffer)
	{
		buffer->Release();
		buffer=NULL;
	}
}

void* MP_INDEX_BUFFER_WRAP::Map(int stride, MP_Device* device)
{
	void* locked_buffer;
	buffer->Lock(0, length, &locked_buffer, 0);
	return locked_buffer;
}

void MP_INDEX_BUFFER_WRAP::Unmap(MP_Device* device)
{
	buffer->Unlock();
}

void MP_INDEX_BUFFER_WRAP::Set(MAGIC_VERTEX_FORMAT* format, int stride, MP_Device* device)
{
	((MP_Device_WRAP*)device)->GetDevice3D()->SetIndices(buffer);
}


// --------------------------------------------------------------------------------

// eng: Class controlling drawing
// rus: ,   
MP_Device_WRAP::MP_Device_WRAP(int width, int height, HWND hwnd) : MP_Device(width, height)
{
	this->hwnd=hwnd;

	direct_3d=NULL;
	device_3d=NULL;
}

// eng: Creating
// rus: 
bool MP_Device_WRAP::Create()
{
	MP_Device::Create();
	
	bool result=false;
	direct_3d=Direct3DCreate9(D3D_SDK_VERSION);
	if (direct_3d)
	{
		bool full_screen=false;

		D3DDISPLAYMODE Display;
		HRESULT ok=direct_3d->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &Display);
		if (ok==D3D_OK)
		{
			D3DPRESENT_PARAMETERS d3d9pp;
			ZeroMemory( &d3d9pp, sizeof(d3d9pp) );
			d3d9pp.SwapEffect=D3DSWAPEFFECT_DISCARD;
			d3d9pp.BackBufferFormat=Display.Format;

			d3d9pp.BackBufferCount=1;
			#ifdef TEST
			d3d9pp.Flags = D3DPRESENTFLAG_LOCKABLE_BACKBUFFER;
			#endif

			d3d9pp.BackBufferWidth=Display.Width;
			d3d9pp.BackBufferHeight=Display.Height;

			#ifdef MAGIC_3D
			if (Magic_IsZBufferEnabled())
			{
				d3d9pp.EnableAutoDepthStencil=TRUE;
				const D3DFORMAT displayFormats[] = {
					D3DFMT_D32,
					D3DFMT_D24X8,
					D3DFMT_D24S8,
					D3DFMT_D24X4S4,
					D3DFMT_D16
				};
				//   
				for(size_t j = 0; j < sizeof(displayFormats) / sizeof(displayFormats[0]); ++j)
				{
					//  
					if (FAILED(direct_3d->CheckDeviceFormat(D3DADAPTER_DEFAULT,
						D3DDEVTYPE_HAL,  D3DFMT_X8R8G8B8,  D3DUSAGE_DEPTHSTENCIL, 
						D3DRTYPE_SURFACE, displayFormats[j]))) {
							continue;
					}
					d3d9pp.AutoDepthStencilFormat = displayFormats[j];
					break;
				}
			}
			#endif

			d3d9pp.PresentationInterval=D3DPRESENT_INTERVAL_IMMEDIATE;

			if (full_screen)
			{
				d3d9pp.Windowed=false;
				d3d9pp.hDeviceWindow=NULL;

				d3d9pp.FullScreen_RefreshRateInHz=Display.RefreshRate;

				window_width=Display.Width;
				window_height=Display.Height;
			}
			else
			{
				d3d9pp.Windowed=true;
				d3d9pp.hDeviceWindow=hwnd;

				RECT client_rect;
				GetClientRect(hwnd,&client_rect);
				window_width=client_rect.right;
				window_height=client_rect.bottom;

				#ifdef TEST
				d3d9pp.BackBufferWidth=client_rect.right;
				d3d9pp.BackBufferHeight=client_rect.bottom;
				#endif
			}

			while (true)
			{
				ok=direct_3d->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hwnd,
					D3DCREATE_HARDWARE_VERTEXPROCESSING,
					&d3d9pp, &device_3d );

				if (ok==D3D_OK)
					break;

				ok=direct_3d->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hwnd,
					D3DCREATE_SOFTWARE_VERTEXPROCESSING,
					&d3d9pp, &device_3d );

				if (ok==D3D_OK)
					break;

				switch (ok)
				{
				case D3DERR_OUTOFVIDEOMEMORY:
					return false;
				case D3DERR_INVALIDCALL:
					// eng: MS-DOS mode
					// rus:  MS-DOS
					Sleep(300);
					break;
				case D3DERR_NOTAVAILABLE:
					// rus:   
					Sleep(300);
					break;
				default:
					Sleep(300);
					break;
				}
			}

			result=true;
		}
	}

	return result;
}

#ifdef MAGIC_3D
// eng: Setting of scene
// rus:  
void MP_Device_WRAP::SetScene3d(bool camera_mode)
{
	MP_Device::SetScene3d(camera_mode);
	// eng: viewport
	// rus: 
	D3DVIEWPORT9 view_port;
	view_port.X=0;
	view_port.Y=0;
	view_port.Width=(unsigned int)window_width;
	view_port.Height=(unsigned int)window_height;
	view_port.MinZ=0.f;
	view_port.MaxZ=1.f;
	device_3d->SetViewport(&view_port);

	D3DXMATRIX world;
	D3DXMatrixIdentity(&world);
	device_3d->SetTransform(D3DTS_WORLD, &world);

	// eng: projection matrix
	// rus:  
	D3DXMATRIX proj;
	float znear=1.f;
	float zfar=1000000.f;
	if (camera_mode)
		D3DXMatrixPerspectiveFovLH(&proj, (float)(MAGIC_PI * 0.15f), ((float)window_width)/((float)window_height), znear, zfar);
	else
		D3DXMatrixOrthoLH(&proj, (float)window_width, (float)window_height, -10000.f, 10000.f);
	device_3d->SetTransform(D3DTS_PROJECTION, &proj);

	MP_POSITION eye(0.f,300.f,-840.f);
	MP_POSITION at(0.f,0.f,0.f);
	MP_POSITION up(0.f,1.f,0.f);

	SetCamera(eye, at, up);
}
#endif

#ifdef MAGIC_3D
// eng: Camera position
// rus:  
void MP_Device_WRAP::SetCamera(MP_POSITION& eye, MP_POSITION& at, MP_POSITION& up)
{
	MP_Device::SetCamera(eye, at, up);

	// eng: view matrix
	// rus:  
	D3DXMATRIX view;
	D3DXVECTOR3 deye(eye.x, eye.y, eye.z);
	D3DXVECTOR3 dat(at.x, at.y, at.z);
	D3DXVECTOR3 dup(up.x, up.y, up.z);

	D3DXMatrixLookAtLH(&view, &deye, &dat, &dup);
	device_3d->SetTransform(D3DTS_VIEW, &view);
}
#endif

// eng: Beginning of scene drawing
// rus:   
void MP_Device_WRAP::BeginScene()
{
	MP_Device::BeginScene();

	DWORD flags=D3DCLEAR_TARGET;
	#ifdef MAGIC_3D
	if (Magic_IsZBufferEnabled())
		flags|=D3DCLEAR_ZBUFFER;
	#endif

	device_3d->Clear(0, NULL, flags, 0, 1.f, 0);

	device_3d->BeginScene();

	// eng: save current states
	// rus:   
	device_3d->GetSamplerState(0,D3DSAMP_ADDRESSU,&state_addressu);
	device_3d->GetSamplerState(0,D3DSAMP_ADDRESSV,&state_addressv);
	device_3d->GetSamplerState(0,D3DSAMP_MAGFILTER,&state_magfilter);
	device_3d->GetSamplerState(0,D3DSAMP_MINFILTER,&state_minfilter);

	device_3d->GetTextureStageState(0, D3DTSS_COLORARG1, &state_colorarg1);
	device_3d->GetTextureStageState(0, D3DTSS_COLORARG2, &state_colorarg2);
	device_3d->GetTextureStageState(0, D3DTSS_COLOROP,   &state_colorop);

	device_3d->GetTextureStageState(0, D3DTSS_ALPHAARG1, &state_alphaarg1);
	device_3d->GetTextureStageState(0, D3DTSS_ALPHAARG2, &state_alphaarg2);
	device_3d->GetTextureStageState(0, D3DTSS_ALPHAOP,   &state_alphaop);

	device_3d->GetRenderState(D3DRS_SRCBLEND, &state_srcblend);
	device_3d->GetRenderState(D3DRS_ALPHABLENDENABLE, &state_alphaenable);

	device_3d->GetRenderState(D3DRS_LIGHTING, &state_lighting);
	device_3d->GetRenderState(D3DRS_ZENABLE, &state_zenable);
	device_3d->GetRenderState(D3DRS_CULLMODE, &state_cullmode);

	// eng: change states
	// rus:  
	device_3d->SetSamplerState(0,D3DSAMP_ADDRESSU,D3DTADDRESS_CLAMP);
	device_3d->SetSamplerState(0,D3DSAMP_ADDRESSV,D3DTADDRESS_CLAMP);
	device_3d->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
	device_3d->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);

	/*
	device_3d->SetTextureStageState (0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
	device_3d->SetTextureStageState (0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
	device_3d->SetTextureStageState (0, D3DTSS_COLOROP,   D3DTOP_MODULATE);

	device_3d->SetTextureStageState( 0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE );
	device_3d->SetTextureStageState( 0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE );
	device_3d->SetTextureStageState( 0, D3DTSS_ALPHAOP,   D3DTOP_MODULATE );

	device_3d->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	device_3d->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
	*/

	#ifdef MAGIC_3D
	if (Magic_IsZBufferEnabled())
		device_3d->SetRenderState(D3DRS_ZFUNC, D3DCMP_LESSEQUAL);
	else
	#endif
	{
		device_3d->SetRenderState(D3DRS_ZENABLE, FALSE);
		device_3d->SetRenderState(D3DRS_ZWRITEENABLE, FALSE);
	}

	device_3d->SetRenderState(D3DRS_LIGHTING, FALSE);

	device_3d->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
}

// eng: End of scene drawing
// rus:   
void MP_Device_WRAP::EndScene()
{
	device_3d->EndScene();

	RECT rect;
	rect.left=0;
	rect.top=0;
	rect.right=window_width;
	rect.bottom=window_height;
	device_3d->Present(&rect, &rect, NULL, NULL);

	// eng: restore states
	// rus:  
	device_3d->SetSamplerState(0, D3DSAMP_ADDRESSU, state_addressu);
	device_3d->SetSamplerState(0, D3DSAMP_ADDRESSV, state_addressv);
	device_3d->SetSamplerState(0, D3DSAMP_MAGFILTER, state_magfilter);
	device_3d->SetSamplerState(0, D3DSAMP_MINFILTER, state_minfilter);

	device_3d->SetTextureStageState(0, D3DTSS_COLORARG1, state_colorarg1);
	device_3d->SetTextureStageState(0, D3DTSS_COLORARG2, state_colorarg2);
	device_3d->SetTextureStageState(0, D3DTSS_COLOROP,   state_colorop);

	device_3d->SetTextureStageState(0, D3DTSS_ALPHAARG1, state_alphaarg1);
	device_3d->SetTextureStageState(0, D3DTSS_ALPHAARG2, state_alphaarg2);
	device_3d->SetTextureStageState(0, D3DTSS_ALPHAOP,   state_alphaop);

	device_3d->SetRenderState(D3DRS_SRCBLEND, state_srcblend);
	device_3d->SetRenderState(D3DRS_ALPHABLENDENABLE, state_alphaenable);

	device_3d->SetRenderState(D3DRS_LIGHTING, state_lighting);
	device_3d->SetRenderState(D3DRS_ZENABLE, state_zenable);
	device_3d->SetRenderState(D3DRS_CULLMODE, state_cullmode);
}

// eng: Indicates that device is lost
// rus:     
bool MP_Device_WRAP::IsLost()
{
	HRESULT hr=device_3d->TestCooperativeLevel();
	return (hr!=D3D_OK);
}

void MP_Device_WRAP::Destroy()
{
	MP_Device::Destroy();

	//vertices.Clear();
	//indexes.Clear();

	if (device_3d)
	{
		device_3d->Release();
		device_3d=NULL;
	}

	if (direct_3d)
	{
		direct_3d->Release();
		direct_3d=NULL;
	}
}

void MP_Device_WRAP::SetVertexFormat(MAGIC_RENDERING_START* start)
{
	int index=layout_list.Select(&(start->format), this);
	MP_LAYOUT_WRAP* layout=(MP_LAYOUT_WRAP*)layout_list.Get(index);
	device_3d->SetVertexDeclaration(layout->declaration);
}

// -------------------------------------------------------------------------------------------

//  
void SetBlendingNormal(LPDIRECT3DDEVICE9 device_3d)
{
	device_3d->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
	device_3d->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	device_3d->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
}

//  
void SetBlendingAdd(LPDIRECT3DDEVICE9 device_3d)
{
	device_3d->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
	device_3d->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	device_3d->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
}

// 
void SetBlendingOpacity(LPDIRECT3DDEVICE9 device_3d)
{
	device_3d->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
}

//   ,     
void (*render_blending[MAGIC_BLENDING_SHADER])(LPDIRECT3DDEVICE9 device_3d)={SetBlendingNormal, SetBlendingAdd, SetBlendingOpacity};

//   
void SetRenderBlending(MP_Device_WRAP* device, MAGIC_RENDER_STATE* s)
{
	LPDIRECT3DDEVICE9 device_3d=device->GetDevice3D();
	render_blending[s->value](device_3d);
}

//    
void SetRenderTextureCount(MP_Device_WRAP* device, MAGIC_RENDER_STATE* s)
{
	LPDIRECT3DDEVICE9 device_3d=device->GetDevice3D();
	device_3d->SetTextureStageState(s->value, D3DTSS_COLOROP, D3DTOP_DISABLE);
}

//   
void SetRenderTexture(MP_Device_WRAP* device, MAGIC_RENDER_STATE* s)
{
	LPDIRECT3DDEVICE9 device_3d=device->GetDevice3D();
	MP_Atlas_WRAP* atlas=(MP_Atlas_WRAP*)device->MP->GetAtlas(s->value);
	LPDIRECT3DTEXTURE9 texture=atlas->GetTexture();
	device_3d->SetTexture(s->index, texture);
}

// ----------------------------------------------------------------

int dx_address[]={D3DTADDRESS_WRAP, D3DTADDRESS_MIRROR, D3DTADDRESS_CLAMP, D3DTADDRESS_BORDER};

//     U
void SetRenderAddressU(MP_Device_WRAP* device, MAGIC_RENDER_STATE* s)
{
	LPDIRECT3DDEVICE9 device_3d=device->GetDevice3D();
	device_3d->SetSamplerState(s->index, D3DSAMP_ADDRESSU, dx_address[s->value]);
}

//     V
void SetRenderAddressV(MP_Device_WRAP* device, MAGIC_RENDER_STATE* s)
{
	LPDIRECT3DDEVICE9 device_3d=device->GetDevice3D();
	device_3d->SetSamplerState(s->index, D3DSAMP_ADDRESSV, dx_address[s->value]);
}

// ----------------------------------------------------------------

int dx_operation[]={D3DTOP_SELECTARG1, D3DTOP_ADD, D3DTOP_SUBTRACT, D3DTOP_MODULATE, D3DTOP_MODULATE2X, D3DTOP_MODULATE4X};

//     RGB
void SetRenderOperationRGB(MP_Device_WRAP* device, MAGIC_RENDER_STATE* s)
{
	LPDIRECT3DDEVICE9 device_3d=device->GetDevice3D();
	device_3d->SetTextureStageState(s->index, D3DTSS_COLOROP,  dx_operation[s->value]);
}

// ----------------------------------------------------------------

int dx_arg[]={D3DTA_CURRENT, D3DTA_DIFFUSE, D3DTA_TEXTURE};

//   1     RGB
void SetRenderArgument1RGB(MP_Device_WRAP* device, MAGIC_RENDER_STATE* s)
{
	LPDIRECT3DDEVICE9 device_3d=device->GetDevice3D();
	device_3d->SetTextureStageState(s->index, D3DTSS_COLORARG1,  dx_arg[s->value]);
}

// ----------------------------------------------------------------

//   2     RGB
void SetRenderArgument2RGB(MP_Device_WRAP* device, MAGIC_RENDER_STATE* s)
{
	LPDIRECT3DDEVICE9 device_3d=device->GetDevice3D();
	device_3d->SetTextureStageState(s->index, D3DTSS_COLORARG2,  dx_arg[s->value]);
}

// ----------------------------------------------------------------

//     A
void SetRenderOperationA(MP_Device_WRAP* device, MAGIC_RENDER_STATE* s)
{
	LPDIRECT3DDEVICE9 device_3d=device->GetDevice3D();
	device_3d->SetTextureStageState(s->index, D3DTSS_ALPHAOP,  dx_operation[s->value]);
}

// ----------------------------------------------------------------

//   1     A
void SetRenderArgument1A(MP_Device_WRAP* device, MAGIC_RENDER_STATE* s)
{
	LPDIRECT3DDEVICE9 device_3d=device->GetDevice3D();
	device_3d->SetTextureStageState(s->index, D3DTSS_ALPHAARG1, dx_arg[s->value]);
}

// ----------------------------------------------------------------

//   2     A
void SetRenderArgument2A(MP_Device_WRAP* device, MAGIC_RENDER_STATE* s)
{
	LPDIRECT3DDEVICE9 device_3d=device->GetDevice3D();
	device_3d->SetTextureStageState(s->index, D3DTSS_ALPHAARG2, dx_arg[s->value]);
}

// ----------------------------------------------------------------

// /   Z-
void SetRenderZEnable(MP_Device_WRAP* device, MAGIC_RENDER_STATE* s)
{
	LPDIRECT3DDEVICE9 device_3d=device->GetDevice3D();
	device_3d->SetRenderState(D3DRS_ZENABLE, s->value);
}

// ----------------------------------------------------------------

//  Z-  
void SetRenderZWrite(MP_Device_WRAP* device, MAGIC_RENDER_STATE* s)
{
	LPDIRECT3DDEVICE9 device_3d=device->GetDevice3D();
	device_3d->SetRenderState(D3DRS_ZWRITEENABLE, s->value);
}

// ----------------------------------------------------------------

//  -
void SetRenderAlphaTestInit(MP_Device_WRAP* device, MAGIC_RENDER_STATE* s)
{
	LPDIRECT3DDEVICE9 device_3d=device->GetDevice3D();
	device_3d->SetRenderState(D3DRS_ALPHAREF, (DWORD)0x00000001);
	device_3d->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATEREQUAL);
}

// ----------------------------------------------------------------

// / -
void SetRenderAlphaTest(MP_Device_WRAP* device, MAGIC_RENDER_STATE* s)
{
	LPDIRECT3DDEVICE9 device_3d=device->GetDevice3D();
	device_3d->SetRenderState(D3DRS_ALPHATESTENABLE, s->value); // / -
}

// ----------------------------------------------------------------

//   ,      
void (*render_states[])(MP_Device_WRAP* device, MAGIC_RENDER_STATE* s)={SetRenderBlending, SetRenderTextureCount, SetRenderTexture, SetRenderAddressU, SetRenderAddressV, SetRenderOperationRGB, SetRenderArgument1RGB, SetRenderArgument2RGB, SetRenderOperationA, SetRenderArgument1A, SetRenderArgument2A, SetRenderZEnable, SetRenderZWrite, SetRenderAlphaTestInit, SetRenderAlphaTest};

void MP_Device_WRAP::SetRenderState(MAGIC_RENDER_STATE* state)
{
	render_states[state->state](this, state);
}

void MP_Device_WRAP::DrawVertices(int starting_index, int indexes_count, int max_vertices)
{
	device_3d->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, 0, max_vertices, starting_index, indexes_count/3);
}


// eng: Creating of layout
// rus:   
MP_LAYOUT* MP_Device_WRAP::NewLayout()
{
	return new MP_LAYOUT_WRAP;
}

// eng: Creating of vertex buffer
// rus:   
MP_BUFFER* MP_Device_WRAP::NewVertexBuffer()
{
	return new MP_VERTEX_BUFFER_WRAP();
}

// eng: Creating of index buffer
// rus:   
MP_BUFFER* MP_Device_WRAP::NewIndexBuffer()
{
	return new MP_INDEX_BUFFER_WRAP();
}

#ifndef MAGIC_3D
int additional_attribute_length[]={0, 8, 0, 0, 0, 0, 0, 0};
int MP_Device_WRAP::GetAdditionalAttributeLength(int attribute, int index)
{
	return additional_attribute_length[attribute];
}
#endif